window.AdminTeamGrid = (function (window, document, $) {
  'use strict';
  var xbox;
  var app = {
    debug: false,
    prefix: 'tgr_',
  };

  //Document Ready
  $(function () {
    app.debug = TGR_PUBLIC_JS.debug_mode === 'on';
    xbox = window.XBOX;
    app.$xbox = $('.xbox');
    app.init();
  });


  app.init = function () {
    app.$settings_form = $('#settings-teamgrid');
    app.$post_body_members = $('body.post-type-teamgrid_members #post-body');
    app.$post_body_teams = $('body.post-type-teamgrid_teams #post-body');
    app.$tab_activation = app.$settings_form.find('.tab-content-activation');

    app.$wp_list_table_members =  $('body.post-type-teamgrid_members .wp-list-table');
    app.$wp_list_table_teams =  $('body.post-type-teamgrid_teams .wp-list-table');

    app.$wp_list_table_members.on('click', '.tgr-duplicate-member', app.duplicate_member);
    app.$wp_list_table_teams.on('click', '.tgr-duplicate-team', app.duplicate_team);

    $('body.wp-admin').on('click', '.tgr-close-message', app.close_info_message);

    //Plugin Activation
    app.update_plugin_activation_status();
    app.$tab_activation.on('click', '#activation-validate-purchase:not(.btn-disabled)', app.update_plugin_status);
    app.$tab_activation.on('focusin', 'input[type="text"]', function (e) {
      $(this).closest('.xbox-field').removeClass('xbox-error');
    });

  };

  app.duplicate_member = function (event) {
    event.preventDefault();
    var $btn = $(this);
    $btn.removeClass('tgr-duplicate-member');
    app.ajax({
      data: {
        ajax_nonce: TGR_ADMIN_JS.ajax_nonce,
        action: 'tgr_duplicate_member',
        member_id: $btn.data('member_id'),
      },
      beforeSend: function () {
        $btn.append("<i class='tgr-icon tgr-icon-spinner tgr-icon-spin'></i>");
      },
      success: function(response){
        c(response);
      },
      complete: function(){
        $btn.find(".tgr-icon-spinner").remove();
        location.reload();
      }
    });
  };

  app.duplicate_team = function (event) {
    event.preventDefault();
    var $btn = $(this);
    $btn.removeClass('tgr-duplicate-team');
    app.ajax({
      data: {
        ajax_nonce: TGR_ADMIN_JS.ajax_nonce,
        action: 'tgr_duplicate_team',
        team_id: $btn.data('team_id'),
      },
      beforeSend: function () {
        $btn.append("<i class='tgr-icon tgr-icon-spinner tgr-icon-spin'></i>");
      },
      success: function(response){
        c(response);
      },
      complete: function(){
        $btn.find(".tgr-icon-spinner").remove();
        location.reload();
      }
    });
  };

  app.update_plugin_activation_status = function () {
    var $status_info = app.$tab_activation.find('.tgr-activation-status');
    var status = app.$tab_activation.find('.xbox-field-id-activation-status .xbox-element').val();
    if (status == 'on') {
      $status_info.alterClass('xbox-color-red', 'xbox-color-green').text('Plugin Activated');
    } else {
      $status_info.alterClass('xbox-color-green', 'xbox-color-red').text('Not Activated');
    }
  };

  app.update_plugin_status = function (event) {
    var $btn = $(this);
    $btn.addClass('btn-disabled');

    // var $section_offer = $('.xbox-section-id-activation-offer');
    // $section_offer.hide();
    var $username = app.$tab_activation.find('.xbox-field-id-activation-username .xbox-element');
    //var $api_key = app.$tab_activation.find('.xbox-field-id-activation-api-key .xbox-element');
    var $purchase_code = app.$tab_activation.find('.xbox-field-id-activation-purchase-code .xbox-element');
    var $email = app.$tab_activation.find('.xbox-field-id-activation-email .xbox-element');
    var type = app.$tab_activation.find('.xbox-field-id-activation-type .xbox-element:checked').val();
    var $domain = app.$tab_activation.find('.xbox-field-id-activation-domain .xbox-element');
    var has_error = false;
    if ($.trim($username.val()).length < 2) {
      $username.closest('.xbox-field').addClass('xbox-error');
      has_error = true;
    }
    //Api key is deprecated
    // if ($.trim($api_key.val()).length < 2) {
    //     $api_key.closest('.xbox-field').addClass('xbox-error');
    //     has_error = true;
    // }
    if ($.trim($purchase_code.val()).length < 2) {
      $purchase_code.closest('.xbox-field').addClass('xbox-error');
      has_error = true;
    }
    if (type == 'deactivation' && $.trim($domain.val()).length < 2) {
      $domain.closest('.xbox-field').addClass('xbox-error');
      has_error = true;
    }

    if (has_error) {
      $btn.removeClass('btn-disabled');
      return;
    }

    var data = {
      ajax_nonce: XBOX_JS.ajax_nonce,
      action: 'tgr_update_plugin_status',
      user_name: $username.val(),
      //api_key: $api_key.val(),//Api key is deprecated
      purchase_code: $purchase_code.val(),
      email: $email.val(),
      domain: $domain.val(),
      type: type,
      auth: app.$tab_activation.find('.xbox-field-id-activation-auth .xbox-element').val(),
    };

    var $xbox_content = $btn.closest('.xbox-content');
    var $status = app.$tab_activation.find('.xbox-field-id-activation-status');
    var $status_info = app.$tab_activation.find('.tgr-activation-status');
    $.ajax({
      type: 'post',
      dataType: 'json',
      url: XBOX_JS.ajax_url,
      data: data,
      beforeSend: function () {
        $xbox_content.find('.tgr-message').remove();
        $xbox_content.append("<i class='tgr-icon tgr-icon-spinner tgr-icon-spin tgr-loader'></i>");
      },
      success: function (response) {
        c('Activation Plugin: response');
        c(response);
        if (!response) {
          return;
        }
        var message = response.message;
        if (response.success) {
          message += ' The changes have already been saved.';
          if (type == 'activation') {
            xbox.set_field_value($status, 'on');
            $status_info.alterClass('xbox-color-red', 'xbox-color-green').text('Plugin Activated');
            //app.show_activation_offer($section_offer, data, response);
          } else {
            if (response.local_deactivation === true) {
              xbox.set_field_value($status, 'off');
              $status_info.alterClass('xbox-color-green', 'xbox-color-red').text('Not Activated');
            }
          }
          $xbox_content.append(app.message('success', false, '', message));
        } else {
          $xbox_content.append(app.message('error', false, '', message));
        }
      },
      error: function (jqXHR, textStatus, errorThrown) {
        cc('Ajax Error, textStatus=', textStatus);
        cc('jqXHR', jqXHR);
        cc('jqXHR.responseText', jqXHR.responseText);
        cc('errorThrown', errorThrown);
        $xbox_content.append(app.message('error', false, '', jqXHR.statusText));
      },
      complete: function (jqXHR, textStatus) {
        $xbox_content.find('.tgr-loader').remove();
        $btn.removeClass('btn-disabled');
      }
    });
  };





  app.set_field_values = function (field_values, set_default) {
    $.each(field_values, function (i, field) {
      field.name = app.parse_field_name(field.name);
      var $field = app.$xbox.find('.xbox-field-id-' + field.name);
      if( $field.length ) {
        xbox.set_field_value($field, field.value, field.unit, set_default);
      }
    });
  };

  app.get_field_value = function (field_name, default_value) {
    return xbox.get_field_value(app.$xbox, field_name, default_value);
  };

  app.parse_field_name = function (field_name) {
    if( field_name.indexOf(app.prefix) == -1 ){
      field_name = app.prefix + field_name;
    }
    return field_name;
  };

  app.close_info_message = function (event) {
    var selector = $(this).hasClass('tgr-close-row') ? '.xbox-row' : '.tgr-message';
    $(this).closest(selector).fadeOut(200, function(){
      $(this).remove();
    });
  };

  app.message = function (type, icon, header, content) {
    var message_class = 'tgr-message tgr-message-' + type;
    if (icon === true) {
      message_class += ' tgr-icon-message';
    }
    var message = '<div class="' + message_class + '">';
    message += '<i class="xbox-icon xbox-icon-remove tgr-close-message"></i>';
    if (header) {
      message += '<header>' + header + '</header>';
    }
    message += '<p>' + content + '</p>';
    message += '</div>';
    return message;
  };

  app.get_unit = function ($target) {
    return $target.closest('.xbox-field').find('input.xbox-unit-number').val();
  };

  app.number_object = function (value) {
    var number = {
      value: value,
      unit: undefined,
    };
    value = value.toString();
    if ($.inArray(value, ['auto', 'initial', 'inherit', 'normal']) > -1) {
      number.value = value;
      number.unit = undefined;
    } else if (value.indexOf('px') > -1) {
      number.value = value.replace('px', '');
      number.unit = 'px';
    } else if (value.indexOf('%') > -1) {
      number.value = value.replace('%', '');
      number.unit = '%';
    } else if (value.indexOf('em') > -1) {
      number.value = value.replace('em', '');
      number.unit = 'em';
    }
    return number;
  };

  app.is_number = function (n) {
    return !isNaN(parseFloat(n)) && isFinite(n);
  };

  app.css = {
    number: function (value, unit) {
      unit = unit || '';
      var arr = ['auto', 'initial', 'inherit', 'normal'];
      if ($.inArray(value, arr) > -1) {
        return value;
      }
      value = value.toString().replace(/[^0-9.\-]/g, '');
      if (this.is_number(value)) {
        return value + unit;
      }
      return 1;
    },
    is_number: function (n) {
      return !isNaN(parseFloat(n)) && isFinite(n);
    },
  };

  app.cookie = {
    set: function (name, value, days) {
      var expires = "";
      if (days) {
        days = parseFloat(days, 10);
        var date = new Date();
        date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
        expires = "; expires=" + date.toGMTString();
      }
      document.cookie = name + "=" + value + expires + "; path=/";
    },
    get: function (name) {
      var nameEQ = name + "=";
      var ca = document.cookie.split(';');
      for (var i = 0; i < ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0) === ' ') {
          c = c.substring(1, c.length);
        }
        if (c.indexOf(nameEQ) === 0) {
          return c.substring(nameEQ.length, c.length);
        }
      }
      return null;
    },
    remove: function (name) {
      this.set(name, "", -1);
    }
  };

  app.ajax = function (options) {
    var defaults = {
      type: 'post',
      data: {
        ajax_nonce: XBOX_JS.ajax_nonce,
      },
      dataType: 'json',
      beforeSend: function () {
      },
      success: function (response) {
      },
      complete: function (jqXHR, textStatus) {
      },
    };
    options = $.extend(true, {}, defaults, options);
    $.ajax({
      url: XBOX_JS.ajax_url,
      type: options.type,
      dataType: options.dataType,
      data: options.data,
      beforeSend: options.beforeSend,
      success: function (response) {
        cc('Ajax Success', response);
        if ($.isFunction(options.success)) {
          options.success.call(this, response);
        }
      },
      error: function (jqXHR, textStatus, errorThrown) {
        cc('Ajax Error, textStatus=', textStatus);
        cc('jqXHR', jqXHR);
        cc('jqXHR.responseText', jqXHR.responseText);
        cc('errorThrown', errorThrown);
      },
      complete: function (jqXHR, textStatus) {
        if ($.isFunction(options.complete)) {
          options.complete.call(this, jqXHR, textStatus);
        }
      }
    });
  };

  app.ajax_example = function () {
    $.ajax({
      type: 'post',
      dataType: 'json',
      url: XBOX_JS.ajax_url,
      data: {
        action: 'tgr_action',
        data: data,
        ajax_nonce: XBOX_JS.ajax_nonce
      },
      beforeSend: function () {
      },
      success: function (response) {
        if (response) {
        }
      },
      error: function (jqXHR, textStatus, errorThrown) {
      },
      complete: function (jqXHR, textStatus) {
      }
    });
  };

  app.get_object_id = function () {
    return $('.xbox').first().data('object-id');
  };

  app.is_empty = function (value) {
    if (value === undefined || value === null) {
      return true;
    } else if (typeof value == 'object') {
      if (value instanceof $) {
        return value.length === 0;
      } else {
        for (var i in value) return false;
        return true;
      }
    } else {
      return (value === false || $.trim(value).length === 0);
    }
  };

  app.isCmdKey = function (e) {
    return !!e.ctrlKey || !!e.metaKey;//Mac support
  };
  app.isShiftKey = function (e) {
    return !!e.shiftKey;
  };

  app.sizeObj = function(obj) {
    var size = 0, key;
    for (key in obj) {
      if (obj.hasOwnProperty(key)) size++;
    }
    return size;
  };

  app.camelToUnderscore = function(text) {
    return text.replace( /([A-Z])/g, "-$1" ).toLowerCase();
  };

  app.isArray = function (obj) {
    return !!obj && obj.constructor === Array;
  };

  app.isObject = function (obj) {
    return !!obj && obj.constructor === Object;
  };

  app.copy_to_clipboard = function (text) {
    var dummy = document.createElement("textarea");
    document.body.appendChild(dummy);
    dummy.value = text;
    dummy.select();
    document.execCommand("copy");
    document.body.removeChild(dummy);
  };

  app.download_json = function(json_string){
    var dataStr = "data:text/json;charset=utf-8," + encodeURIComponent(json_string);
    var downloadAnchorNode = document.createElement('a');
    downloadAnchorNode.setAttribute("href",     dataStr);
    downloadAnchorNode.setAttribute("download", exportName + ".json");
    document.body.appendChild(downloadAnchorNode); // required for firefox
    downloadAnchorNode.click();
    downloadAnchorNode.remove();
  };

  app.getParamsAsObject = function (query) {
    if( ! query ){
      return {};
    }
    query = query.substring(query.indexOf('?') + 1);

    var re = /([^&=]+)=?([^&]*)/g;
    var decodeRE = /\+/g;

    var decode = function (str) {
      return decodeURIComponent(str.replace(decodeRE, " "));
    };

    var params = {}, e;
    while (e = re.exec(query)) {
      var k = decode(e[1]), v = decode(e[2]);
      if (k.substring(k.length - 2) === '[]') {
        k = k.substring(0, k.length - 2);
        (params[k] || (params[k] = [])).push(v);
      }
      else params[k] = v;
    }

    var assign = function (obj, keyPath, value) {
      var lastKeyIndex = keyPath.length - 1;
      for (var i = 0; i < lastKeyIndex; ++i) {
        var key = keyPath[i];
        if (!(key in obj))
          obj[key] = {}
        obj = obj[key];
      }
      obj[keyPath[lastKeyIndex]] = value;
    }

    for (var prop in params) {
      var structure = prop.split('[');
      if (structure.length > 1) {
        var levels = [];
        structure.forEach(function (item, i) {
          var key = item.replace(/[?[\]\\ ]/g, '');
          levels.push(key);
        });
        assign(params, levels, params[prop]);
        delete(params[prop]);
      }
    }
    return params;
  };

  app.scroll_to = function ($this, delay, offset, callback) {
    offset = offset >= 0 ? offset : 300;
    delay = delay || 650;
    $('html,body').stop().animate({ scrollTop: Math.abs($this.offset().top - offset) }, delay, 'swing', callback);
    return false;
  };

  //Funciones privadas
  function get_class_starts_with($elment, starts_with) {
    return $.grep($elment.attr('class').split(" "), function (v, i) {
      return v.indexOf(starts_with) === 0;
    }).join();
  }

  //Debug
  function c(msg) {
    console.log(msg);
  }

  function cc(msg, msg2) {
    console.log(msg, msg2);
  }

  function clog(msg, msg2) {
    if (app.debug) {
      if( msg2 ){
        console.log(msg, msg2);
      } else {
        console.log(msg);
      }
    }
  }

  return app;

})(window, document, jQuery);

jQuery.fn.getValidationMessages = function (fields) {
  var message = "";
  var name = "";
  fields = fields || 'input, textarea';
  this.each(function () {
    $(this).find(fields).each(function (index, el) {
      if (el.checkValidity() === false) {
        name = $("label[for=" + el.id + "]").html() || el.placeholder || el.name || el.id;
        message = message + name + ": " + (this.validationMessage || 'Invalid value.') + "\n";
      }
    });

  });
  return message;
};


jQuery.fn.infoData = function () {
  var data = {};
  [].forEach.call(this.get(0).attributes, function (attr) {
    if (/^data-/.test(attr.name)) {
      var camelCaseName = attr.name.substr(5).replace(/-(.)/g, function ($0, $1) {
        return $1.toUpperCase();
      });
      data[camelCaseName] = attr.value;
    }
  });
  return data;
}