<?php namespace TeamGrid\Includes;


class AssetsLoader {
    public $plugin;
    public $admin_url = '';
    public $public_url = '';
    protected static $instance = null;

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    private function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->admin_url = TGR_URL . 'assets/admin/';
        $this->public_url = TGR_URL . 'assets/public/';
        $this->libs_url = TGR_URL . 'libs/';

        add_action( 'admin_enqueue_scripts', array( $this, 'add_admin_scripts' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'add_admin_styles' ) );
        add_action( 'wp_enqueue_scripts', array( $this, 'add_public_scripts' ) );
        add_action( 'wp_enqueue_scripts', array( $this, 'add_public_styles' ) );

        add_filter( "autoptimize_filter_js_exclude", array( $this, 'autoptimize_filter_js_exclude' ), 10, 1 );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add admin scripts
    |---------------------------------------------------------------------------------------------------
    */
    public function add_admin_scripts( $hook ){
        if( ! $this->should_add() ){
            return;
        }

        //Wordpress scripts
        $deps_scripts = array( 'jquery' );
        if( function_exists( 'wp_enqueue_media') ){
            wp_enqueue_media();
        } else{
            wp_enqueue_script( 'media-upload' );
        }


        wp_register_script( 'tgr-admin-js', $this->admin_url . 'js/tgr-admin.js', $deps_scripts, TGR_VERSION );
        wp_enqueue_script( 'tgr-admin-js' );

        //Plugin scripts
        wp_register_script( 'tgr-designs', $this->admin_url . 'js/tgr-designs.js', array( 'tgr-admin-js' ), TGR_VERSION );
        wp_enqueue_script( 'tgr-designs' );

        wp_register_script( 'tgr-team-editor', $this->admin_url . 'js/tgr-team-editor.js', array( 'tgr-admin-js' ), TGR_VERSION );
        wp_enqueue_script( 'tgr-team-editor' );

        wp_register_script( 'tgr-team-onchange', $this->admin_url . 'js/tgr-team-onchange.js', array( 'tgr-admin-js' ), TGR_VERSION );
        wp_enqueue_script( 'tgr-team-onchange' );

        wp_register_script( 'tgr-member-editor', $this->admin_url . 'js/tgr-member-editor.js', array( 'tgr-admin-js' ), TGR_VERSION );
        wp_enqueue_script( 'tgr-member-editor' );

        wp_localize_script( 'tgr-admin-js', 'TGR_ADMIN_JS', $this->admin_localization() );

    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add admin styles
    |---------------------------------------------------------------------------------------------------
    */
    public function add_admin_styles( $hook ){
        wp_register_style( 'tgr-admin-css', $this->admin_url . 'css/tgr-admin.css', array(), TGR_VERSION );
        wp_enqueue_style( 'tgr-admin-css' );

        if( ! $this->should_add() ){
            return;
        }

        $this->add_public_styles();
        $this->add_public_scripts();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if scripts and styles should be added
    |---------------------------------------------------------------------------------------------------
    */
    private function should_add(){
        global $current_screen;
        $load_in = array(
            $this->plugin->arg( 'post_types', 'members' ),
            $this->plugin->arg( 'post_types', 'teams' ),
        );
        if( in_array( $current_screen->post_type, $load_in ) ){
            return true;
        }
        return false;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add public scripts
    |---------------------------------------------------------------------------------------------------
    */
    public function add_public_scripts(){
        //Wordpress scripts
        $deps_scripts = array( 'jquery', 'jquery-effects-core', 'jquery-effects-slide' );

        //Plugin scripts
        wp_register_script( 'team-grid', $this->public_url . 'js/team-grid.js', $deps_scripts, TGR_VERSION );
        wp_enqueue_script( 'team-grid' );

        wp_register_script( 'tgr-jquery-animatecss', $this->libs_url . 'jquery.animatecss/jquery.animatecss.js', array( 'team-grid' ), TGR_VERSION );
        wp_enqueue_script( 'tgr-jquery-animatecss' );

        wp_register_script( 'tgr-scroll', $this->libs_url . 'nanoScrollerJS/jquery.nanoscroller.js', array( 'team-grid' ), TGR_VERSION );
        wp_enqueue_script( 'tgr-scroll' );

        wp_localize_script( 'team-grid', 'TGR_PUBLIC_JS', $this->public_localization() );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add public styles
    |---------------------------------------------------------------------------------------------------
    */
    public function add_public_styles(){
        wp_register_style( 'team-grid', $this->public_url . 'css/team-grid.css', array(), TGR_VERSION );
        wp_enqueue_style( 'team-grid' );

        wp_register_style( 'tgr-font-awesome', $this->public_url . 'css/fontawesome-team-grid.css', array(), TGR_VERSION );
        wp_enqueue_style( 'tgr-font-awesome' );

        wp_register_style( 'tgr-animate', $this->libs_url . 'animate/animate.css', array(), TGR_VERSION );
        wp_enqueue_style( 'tgr-animate' );

        wp_register_style( 'tgr-scroll', $this->libs_url . 'nanoScrollerJS/nanoscroller.css', array(), TGR_VERSION );
        wp_enqueue_style( 'tgr-scroll' );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | WP admin_Localization
    |---------------------------------------------------------------------------------------------------
    */
    public function admin_localization(){
        $l10n = array(
            'version' => TGR_VERSION,
            'is_admin' => is_admin(),
            'home_url' => home_url(),
            'admin_url' => admin_url(),
            'plugin_url' => TGR_URL,
            'ajax_url' => admin_url( 'admin-ajax.php' ),

            'ajax_nonce' => wp_create_nonce( 'tgr_admin_ajax_nonce' ),

            'taxonomies' => $this->plugin->arg( 'taxonomies' ),
            'texts' => array(
                'saving_changes' => esc_html__( 'Saving changes', 'teamgrid' ),
                'please_wait' => esc_html__( 'Please wait a moment', 'teamgrid' ),
                'confirm' => esc_html__( 'Confirm', 'teamgrid' ),
                'use_design_content' => esc_html__( 'Only styles such as colors, borders, and text sizes will be replaced.', 'teamgrid' ),
            )
        );
        return $l10n;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | WP public_Localization
    |---------------------------------------------------------------------------------------------------
    */
    public function public_localization(){
        $l10n = array(
            'version' => TGR_VERSION,
            'is_admin' => is_admin(),
            'home_url' => home_url(),
            'admin_url' => admin_url(),
            'plugin_url' => TGR_URL,
            'ajax_url' => admin_url( 'admin-ajax.php' ),
            'ajax_nonce' => wp_create_nonce( 'tgr_ajax_nonce' ),
            'debug_mode' => $this->plugin->settings->option('debug-mode'),

            'google_fonts' => array_values( Assets::google_fonts() ),
            'animation_effects' => Assets::animation_effects(),
            'filters' =>  \XboxItems::terms( $this->plugin->arg( 'taxonomies', 'filters' ), array( 'hide_empty' => true ) ),
            'texts' => array(
                'allMembers' => $this->plugin->settings->option('filter-text-all'),
                'memberImage' => $this->plugin->settings->option('table-text-image'),
                'memberName' => $this->plugin->settings->option('table-text-name'),
                'memberRole' => $this->plugin->settings->option('table-text-role'),
                'memberDescription' => $this->plugin->settings->option('table-text-description'),
                'memberSocialIcons' => $this->plugin->settings->option('table-text-social-icons'),
                'memberContent' => $this->plugin->settings->option('table-text-content'),
                'memberFields' => $this->plugin->settings->option('table-text-fields'),
                'memberSkills' => $this->plugin->settings->option('table-text-skills'),
            ),
            'layouts' => TeamOptions::$layouts,
            'generalSettings' => array(
                'socialIcons' => array(
                    'title' => $this->plugin->settings->option('social-icons-title'),
                ),
                'fields' => array(
                    'title' => $this->plugin->settings->option('fields-title'),
                ),
                'skills' => array(
                    'title' => $this->plugin->settings->option('skills-title'),
                ),
            ),
        );
        return $l10n;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Exclude js files for Autoptimize plugin
    |---------------------------------------------------------------------------------------------------
    */
    public function autoptimize_filter_js_exclude( $exclude ){
        return $exclude.", js/team-grid.js, js/team-grid.min.js";
    }

}
