<?php namespace TeamGrid\Includes;


class Member extends MemberOptions {
    protected static $instance = null;
    public $plugin;

    public $team = null;
    public $publish_date = '';
    public $attachment_id = 0;
    public $has_thumbnail = false;
    public $thumbnail = '';
    public $member_image = '';
    public $index = -1;
    public $permalink = '';
    public $filters = array();
    public $filters_string = '';
    public $filters_ids = array();
    public $fields = array();
    public $fields_string = '';
    public $fields_ids = array();
    public $skills = array();
    public $skills_string = '';
    public $skills_ids = array();


    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    public function __construct( $options = array() ){
        $this->plugin = Functions::get_plugin_instance();

        parent::__construct( $options );

        $this->init();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Init
    |---------------------------------------------------------------------------------------------------
    */
    private function init(){
        $this->init_member_image();
        $this->init_filters();
        $this->init_fields();
        $this->init_skills();
        $this->publish_date = $this->option( 'publish_date' );
        $this->permalink = get_permalink( $this->id );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the member's main image or a default image
    |---------------------------------------------------------------------------------------------------
    */
    public function get_image( $default = '' ){
        if( $this->member_image ){
            return $this->member_image;
        }
        if( $default ){
            return $default;
        }
        return TGR_ASSETS_ADMIN_URL . 'images/default-member-image.jpg';

    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the member's thumbnail image or a default image
    |---------------------------------------------------------------------------------------------------
    */
    public function get_thumbnail( $default = '' ){
        if( $this->has_thumbnail ){
            return $this->thumbnail;
        }
        if( $default ){
            return $default;
        }
        return TGR_ASSETS_ADMIN_URL . 'images/default-member-thumbnail.jpg';

    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Initialize the images
    |---------------------------------------------------------------------------------------------------
    */
    public function init_member_image(){
        $saved_image = $this->get_option( 'image' );
        $this->attachment_id = attachment_url_to_postid( $saved_image );
        $resized_image = $this->get_resized_image();

        //Thumbnail
        $thumb = wp_get_attachment_image_src( $this->attachment_id, 'thumbnail' );
        if( is_array( $thumb ) ){
            $this->has_thumbnail = true;
            $this->thumbnail = $thumb[0];
        }

        if( ! $resized_image ){
            $resized_image = $saved_image;
        }

        $this->member_image = $resized_image;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the main image in its correct dimensions
    |---------------------------------------------------------------------------------------------------
    */
    private function get_resized_image(){
        $resized_image = '';
        $image_size_name = $this->option( 'image-resizing-name' );
        $plugin_image_sizes = Settings::get_image_sizes();
        $wp_image_sizes = Functions::get_image_sizes();

        if( isset( $wp_image_sizes[$image_size_name] ) ){
            $image_size = array(
                'name' => $image_size_name,
                'width' => (int) $wp_image_sizes[$image_size_name]['width'],
                'height' => (int) $wp_image_sizes[$image_size_name]['height'],
            );
        } else{
            $image_size = array(
                'name' => $plugin_image_sizes['1']['name'],
                'width' => (int) $plugin_image_sizes['1']['width'],
                'height' => (int) $plugin_image_sizes['1']['height'],
            );
        }

        $image_by_name = wp_get_attachment_image_src( $this->attachment_id, $image_size['name'] );
        $image_by_size = wp_get_attachment_image_src( $this->attachment_id, array( $image_size['width'], $image_size['height'] ) );

        //We check if there is an image using the image size name
        $image_source = '';
        if( is_array( $image_by_name ) ){
            $image_source = $image_by_name[0];
            if( $image_by_name[3] == true || ( $image_by_name[1] == $image_size['width'] && $image_by_name[2] == $image_size['height'] ) ){
                $resized_image = $image_by_name[0];
            } else if( is_array( $image_by_size ) ){
                //Check if there is an image with the required dimensions.
                //It is also true when there is an image with a larger size but with the same required proportions
                //For example: size is required: 200x200 and the image exists path/image-400x400.jpg
                if( $image_by_size[3] == true || ( $image_by_size[1] == $image_size['width'] && $image_by_size[2] == $image_size['height'] ) ){
                    $resized_image = $image_by_size[0];
                }
            }
        }

        //We check if the image has the same proportions as required
        if( $resized_image ){
            $uploads = wp_upload_dir();
            $file_path = str_replace( $uploads['baseurl'], $uploads['basedir'], $resized_image );
            if( $real_image_size = @getimagesize( $file_path ) ){
                $width = $real_image_size[0];
                $height = $real_image_size[1];
                $ration_required_size = round( $image_size['width'] / $image_size['height'], 2 );
                $ration_real_size = round( $width / $height, 2 );
                if( $ration_required_size != $ration_real_size ){
                    $resized_image = '';
                    $image_source = $this->option( 'image' );
                }
            }
        }

        //We check if the image exists in the file list and create the new resized image
        if( ! $resized_image && $this->attachment_id && $image_source ){
            $uploads = wp_upload_dir();
            $file_url = $image_source;
            $file_path = str_replace( $uploads['baseurl'], $uploads['basedir'], $file_url );
            $img_editor = wp_get_image_editor( $file_path );
            if( ! is_wp_error( $img_editor ) && stripos( $file_path, 'http' ) === false ){
                $img_editor->resize( $image_size['width'], $image_size['height'], true );
                $file_url = $img_editor->generate_filename();
                $file_path = str_replace( $uploads['baseurl'], $uploads['basedir'], $file_url );
                if( ! file_exists( $file_path ) ){
                    $saved = $img_editor->save( $file_path );
                    if( is_array( $saved ) ){
                        $resized_image = str_replace( $uploads['basedir'], $uploads['baseurl'], $saved['path'] );
                    }
                } else{
                    $resized_image = str_replace( $uploads['basedir'], $uploads['baseurl'], $file_path );
                }
            }
        }
        return $resized_image;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Initialize the member's filters
    |---------------------------------------------------------------------------------------------------
    */
    public function init_filters(){
        $object_terms = wp_get_object_terms( $this->id, $this->plugin->arg( 'taxonomies', 'filters' ) );
        if( ! is_wp_error( $object_terms ) ){
            foreach( $object_terms as $term ){
                $this->filters[] = $term->slug;
                $this->filters_ids[] = $term->term_id;
            }
            $this->filters_string = esc_attr( implode( ', ', $this->filters ) );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Initialize the member's fields
    |---------------------------------------------------------------------------------------------------
    */
    public function init_fields(){
        $object_terms = wp_get_object_terms( $this->id, $this->plugin->arg( 'taxonomies', 'fields' ) );
        if( ! is_wp_error( $object_terms ) ){
            foreach( $object_terms as $term ){
                $this->fields[] = $term->slug;
                $this->fields_ids[] = $term->term_id;
            }
            $this->fields_string = esc_attr( implode( ', ', $this->fields ) );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Initialize the member's skills
    |---------------------------------------------------------------------------------------------------
    */
    public function init_skills(){
        $object_terms = wp_get_object_terms( $this->id, $this->plugin->arg( 'taxonomies', 'skills' ) );
        if( ! is_wp_error( $object_terms ) ){
            foreach( $object_terms as $term ){
                $this->skills[] = $term->slug;
                $this->skills_ids[] = $term->term_id;
            }
            $this->skills_string = esc_attr( implode( ', ', $this->skills ) );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Set team
    |---------------------------------------------------------------------------------------------------
    */
    public function set_team( $team ){
        $this->team = $team;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Render html
    |---------------------------------------------------------------------------------------------------
    */
    public function render( $index = -1 ){
        $this->index = $index;

        $return = '';
        if( $this->team->is_layout_table() ){
            $return .= $this->build_item_for_table_layout();
        } else{
            $return .= $this->build_item();
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build html for the layout table
    |---------------------------------------------------------------------------------------------------
    */
    public function build_item_for_table_layout(){
        $return = '';
        $table = $this->team->get_table_options();

        $return .= "<div class='tgr-tr tgr-item' data-id='" . esc_attr( $this->id ) . "' data-index='" . esc_attr( $this->index ) . "' data-filters='" . esc_attr( $this->filters_string ) . "'>";

        if( in_array( 'image', $table['columns'] ) ){
            $return .= "<div class='tgr-td tgr-item-inner'>";
            $return .= "<div class='tgr-item-image'>";
            $return .= $this->build_item_image();
            $return .= "</div>";
            $return .= "</div>";
        }
        if( in_array( 'name', $table['columns'] ) ){
            $return .= "<div class='tgr-td tgr-item-inner'>";
            $return .= $this->build_item_title();
            $return .= "</div>";
        }
        if( in_array( 'role', $table['columns'] ) ){
            $return .= "<div class='tgr-td tgr-item-inner'>";
            $return .= $this->build_item_subtitle();
            $return .= "</div>";
        }
        if( in_array( 'description', $table['columns'] ) ){
            $return .= "<div class='tgr-td tgr-item-inner'>";
            $return .= $this->build_item_description();
            $return .= "</div>";
        }
        if( in_array( 'social', $table['columns'] ) ){
            $return .= "<div class='tgr-td tgr-item-inner'>";
            $return .= $this->build_item_social();
            $return .= "</div>";
        }
        if( in_array( 'fields', $table['columns'] ) ){
            $return .= "<div class='tgr-td tgr-item-inner'>";
            $return .= $this->build_member_fields();
            $return .= "</div>";
        }
        if( in_array( 'skills', $table['columns'] ) ){
            $return .= "<div class='tgr-td tgr-item-inner'>";
            $return .= $this->build_member_skills();
            $return .= "</div>";
        }

        $return .= "</div>";//tgr-tr
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Render item
    |---------------------------------------------------------------------------------------------------
    */
    public function build_item(){
        $return = '';
        $item_class = 'tgr-item';
        $item_inner_class = 'tgr-item-inner tgr-animate-social-icons';
        if( $this->team->is_layout_flip_card() ){
            $item_inner_class .= ' tgr-flip-card-container';
        }
        if( ! $this->team->is_layout_list() ){
            $item_class .= ' tgr-grid-item';
        }
        $return .= "<div class='" . esc_attr( $item_class ) . "' data-id='" . esc_attr( $this->id ) . "' data-index='" . esc_attr( $this->index ) . "' data-filters='" . esc_attr( $this->filters_string ) . "'>";
        $return .= "<div class='" . esc_attr( $item_inner_class ) . "'>";
        if( $this->team->is_layout_list() ){
            $return .= $this->build_item_for_list_layout();
        } else{
            $return .= $this->build_item_for_grid_layout();
        }
        $return .= "</div>";//tgr-item-inner
        $return .= "</div>";//tgr-item
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build html for member shortcode
    |---------------------------------------------------------------------------------------------------
    */
    public function display(){
        $return = '';
        $return .= "<div class='teamgrid-member'>";
        $return .= $this->build_member_image();
        $return .= $this->build_member_header();
        $return .= $this->build_member_content();
        $return .= $this->build_member_fields();
        $return .= $this->build_member_skills();
        $return .= $this->build_member_social();
        $return .= "</div>";//teamgrid-member
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build html for the layout list
    |---------------------------------------------------------------------------------------------------
    */
    public function build_item_for_list_layout(){
        $return = '';
        $return .= "<div class='tgr-member'>";
        $return .= "<div class='tgr-flexbox'>";
        $return .= "<div class='tgr-left'>";
        $return .= $this->build_member_image();
        $return .= "</div>";//tgr-left
        $return .= "<div class='tgr-right'>";
        $return .= $this->build_member_header();
        $return .= $this->build_member_content();
        $return .= $this->build_member_fields();
        $return .= $this->build_member_skills();
        $return .= $this->build_member_social();
        $return .= "</div>";//tgr-right
        $return .= "</div>";//tgr-flexbox
        $return .= "</div>";//tgr-member
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build html for grid layouts
    |---------------------------------------------------------------------------------------------------
    */
    public function build_item_for_grid_layout(){
        $return = '';
        $figure_class = 'tgr-figure';
        if( $this->team->is_layout_flip_card() ){
            $figure_class .= ' tgr-flip-card';
        }
        $return .= "<figure class='" . esc_attr( $figure_class ) . "'>";

        if( $this->team->is_layout_flip_card() ){
            $return .= "<div class='tgr-flip-front'>";
        }

        $return .= "<div class='tgr-item-image'>";
        $return .= $this->build_item_image();
        $return .= $this->build_item_header();
        $return .= $this->build_item_social();
        $return .= $this->build_item_description();
        $return .= "</div>";//tgr-item-image

        $return .= "<figcaption class='tgr-figcaption'>";
        $return .= $this->build_item_header();
        $return .= $this->build_item_social();
        $return .= $this->build_item_description();
        $return .= "</figcaption>";

        if( $this->team->is_layout_flip_card() ){
            $return .= "</div>";
            $return .= "<div class='tgr-flip-back'>";
            $return .= "<figcaption class='tgr-figcaption'>";
            $return .= $this->build_item_image();
            $return .= $this->build_item_header();
            $return .= $this->build_item_social();
            $return .= $this->build_item_description();
            $return .= "</figcaption>";
            $return .= "</div>";//tgr-flip-back
        }

        $return .= "</figure>";

        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_item_header
    |---------------------------------------------------------------------------------------------------
    */
    public function build_member_image(){
        $return = '';
        $return .= "<div class='tgr-member-image'>";
        $return .= "<img src='" . esc_url( $this->get_image() ) . "'>";
        $return .= "</div>";//tgr-member-image
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_item_header
    |---------------------------------------------------------------------------------------------------
    */
    public function build_item_header(){
        $return = '';
        $return .= "<div class='tgr-item-header'>";
        $title_after_role = $this->team ? $this->team->option( 'member-title-after-subtitle' ) == 'on' : false;
        if( $title_after_role ){
            $return .= $this->build_item_subtitle();
            $return .= $this->build_item_title();
        } else{
            $return .= $this->build_item_title();
            $return .= $this->build_item_subtitle();
        }
        $return .= "</div>";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_member_header
    |---------------------------------------------------------------------------------------------------
    */
    public function build_member_header(){
        $return = '';
        $return .= "<div class='tgr-member-header'>";
        $title_after_role = $this->team ? $this->team->option( 'member-title-after-subtitle' ) == 'on' : false;
        if( $title_after_role ){
            $return .= $this->build_member_role();
            $return .= $this->build_member_title();
        } else{
            $return .= $this->build_member_title();
            $return .= $this->build_member_role();
        }
        $return .= "</div>";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_item_image
    |---------------------------------------------------------------------------------------------------
    */
    public function build_item_image(){
        $return = '';
        if( $this->team->option( 'layout-hide-image' ) == 'off' ){
            $return .= "<img class='tgr-img' src='" . esc_url( $this->get_image() ) . "'>";
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_item_title
    |---------------------------------------------------------------------------------------------------
    */
    public function build_item_title(){
        $return = "<div class='tgr-item-title'>";
        $title_tag = $this->team ? $this->team->option( 'member-title-tag' ) : 'h3';
        $title_tag = esc_attr( $title_tag );
        $return .= "<$title_tag>$this->title</$title_tag>";
        $return .= "</div>";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_member_title
    |---------------------------------------------------------------------------------------------------
    */
    public function build_member_title(){
        $return = '';
        $return .= "<div class='tgr-member-title'>";
        $title_tag = $this->team ? $this->team->option( 'member-title-tag' ) : 'h3';
        $title_tag = esc_attr( $title_tag );
        $return .= "<$title_tag>$this->title</$title_tag>";
        $return .= "</div>";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_item_subtitle
    |---------------------------------------------------------------------------------------------------
    */
    public function build_item_subtitle(){
        $return = "<div class='tgr-item-subtitle'>";
        $subtitle_tag = $this->team ? $this->team->option( 'member-subtitle-tag' ) : 'h4';
        $subtitle_tag = esc_attr( $subtitle_tag );
        $return .= "<$subtitle_tag>{$this->option('role')}</$subtitle_tag>";
        $return .= "</div>";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_member_role
    |---------------------------------------------------------------------------------------------------
    */
    public function build_member_role(){
        $return = '';
        $return .= "<div class='tgr-member-role'>";
        $subtitle_tag = $this->team ? $this->team->option( 'member-subtitle-tag' ) : 'h4';
        $subtitle_tag = esc_attr( $subtitle_tag );
        $return .= "<$subtitle_tag>{$this->option('role')}</$subtitle_tag>";
        $return .= "</div>";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_item_description
    |---------------------------------------------------------------------------------------------------
    */
    public function build_item_description(){
        $return = "<div class='tgr-item-desc'>";
        $return .= $this->get_member_description();
        $return .= "</div>";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_member_content
    |---------------------------------------------------------------------------------------------------
    */
    public function build_member_content(){
        $return = "<div class='tgr-member-content'>";
        $return .= $this->get_member_content();
        $return .= "</div>";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_item_social
    |---------------------------------------------------------------------------------------------------
    */
    public function build_item_social(){
        $return = "<ul class='tgr-item-social'>";
        $social = $this->get_social();
        foreach( $social as $link ){
            $return .= "<li><a href='" . esc_url( $link['url'] ) . "' target='_blank' class='tgr-item-link tgr-animation-bounce-in' title='" . esc_attr( $link['title'] ) . "'>";
            if($link['custom_icon']){
                $return .= $link['custom_icon'];
            } else {
                $return .= "<i class='" . esc_attr( $link['icon'] ) . "'></i>";
            }
            $return .= "</a></li>";
        }
        $return .= "</ul>";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_member_social
    |---------------------------------------------------------------------------------------------------
    */
    public function build_member_social(){
        $return = "<div class='tgr-member-social'>";
        $social = $this->get_social();
        if( Functions::is_empty( $social ) ){
            return '';
        }
        $title = $this->plugin->settings->option( 'social-icons-title' );
        if( $title ){
            $return .= $title;
        }
        $return .= "<ul>";
        foreach( $social as $link ){
            $return .= "<li><a href='" . esc_url( $link['url'] ) . "' target='_blank' class='tgr-member-link tgr-animation-bounce-in' title='" . esc_attr( $link['title'] ) . "'>";
            if($link['custom_icon']){
                $return .= $link['custom_icon'];
            } else {
                $return .= "<i class='" . esc_attr( $link['icon'] ) . "'></i>";
            }
            $return .= "</a></li>";
        }
        $return .= "</ul></div>";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Render the fields
    |---------------------------------------------------------------------------------------------------
    */
    public function build_member_fields(){
        $return = "<div class='tgr-member-fields'>";
        $fields = $this->get_member_fields();
        if( Functions::is_empty( $fields ) ){
            return '';
        }
        $title = $this->plugin->settings->option( 'fields-title' );
        if( $title ){
            $return .= $title;
        }
        foreach( $fields as $field ){
            $return .= "<p>";
            if( $field['icon'] ){
                $return .= "<i class='" . esc_attr( $field['icon'] ) . "'></i>";
            }
            if( $field['name'] ){
                $return .= "<span>" . esc_html( $field['name'] ) . "</span>";
            }
            $return .= ": ";
            if( $field['url_type'] == 'auto' ){
                if( Functions::is_email( $field['value'] ) ){
                    $return .= "<a href='mailto:" . esc_attr( $field['value'] ) . "' class='tgr-member-link'>" . esc_html( $field['value'] ) . "</a>";
                } else{
                    $return .= $field['value'];
                }
            } else{
                $return .= "<a href='" . esc_url( $field['url'] ) . "' target='" . esc_attr( $field['target'] ) . "' class='tgr-member-link'>" . esc_html( $field['value'] ) . "</a>";
            }
            $return .= "</p>";
        }
        $return .= "</div>";//.tgr-member-fields
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Render skills
    |---------------------------------------------------------------------------------------------------
    */
    public function build_member_skills(){
        $return = "<div class='tgr-member-skills'>";
        $skills = $this->get_member_skills();
        if( Functions::is_empty( $skills ) ){
            return '';
        }
        $title = $this->plugin->settings->option( 'skills-title' );
        if( $title ){
            $return .= $title;
        }
        foreach( $skills as $skill ){
            $show_percent = $this->team ? $this->team->option('skills-show-percent') : 'on';
            $percentage = $skill['percent'] . ( $show_percent == 'on' ? '%' : '');
            if( $skill['percent'] == '' || $skill['percent'] == '0' ){
                $percentage = '';
            }
            $return .= "<div class='tgr-member-skill'>";
            $return .= "<span class='tgr-skill-name'>" . esc_html( $skill['name'] . " " . $percentage );
            if( $skill['time'] ){
                $return .= esc_attr( " - {$skill['time']}" );
            }
            $return .= "</span>";
            $return .= "<div class='tgr-skill-bg'><span class='tgr-skill-percent' style='width: " . esc_attr( $skill['percent'] ) . "%'></span></div>";
            $return .= "</div>";
        }
        $return .= "</div>";//.tgr-member-skills
        return $return;
    }

}