<?php namespace TeamGrid\Includes;

class PluginLoader {
	public $plugin;
	protected static $instance = null;

	/*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
	private function __construct( $plugin ) {
		$this->plugin = $plugin;

		$this->constants();
		$this->hooks();
		$this->general_files();
		$this->assets_loader();
		$this->general_settings();
		$this->prepare_editor();
		$this->include_xbox_framework();
		$this->ajax();
	}

	/*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
	private function __clone(){}//Stopping Clonning of Object
	public function __wakeup(){}//Stopping unserialize of object

	public static function get_instance( $plugin = null ) {
		if ( null === self::$instance ) {
			self::$instance = new self( $plugin );
		}
		return self::$instance;
	}

	/*
	|---------------------------------------------------------------------------------------------------
	| Constants
	|---------------------------------------------------------------------------------------------------
	*/
	public function constants(){
		define( 'TGR_PREFIX',    $this->plugin->arg( 'prefix' ) );
		define( 'TGR_VERSION',    $this->plugin->arg( 'version' ) );
		define( 'TGR_SLUG',       $this->plugin->arg( 'slug' ) );
		define( 'TGR_TEXT_DOMAIN', $this->plugin->arg( 'text_domain' ) );
		define( 'TGR_DIR',        trailingslashit( dirname( dirname( __FILE__ ) ) ) );
        define( 'TGR_OPTIONS_DIR',  trailingslashit( TGR_DIR.'includes/options/' ) );
        define( 'TGR_URL',        trailingslashit( plugins_url( '', dirname( __FILE__ ) ) ) );
        define( 'TGR_ASSETS_URL',  trailingslashit(TGR_URL.'assets') );
        define( 'TGR_ASSETS_ADMIN_URL',  trailingslashit(TGR_URL.'assets/admin') );
        define( 'TGR_ASSETS_PUBLIC_URL',  trailingslashit(TGR_URL.'assets/public') );
        define( 'TGR_BREAKPOINT_EXTRALARGE', 1560 );
        define( 'TGR_BREAKPOINT_LARGE', 1280 );
        define( 'TGR_BREAKPOINT_LAPTOP', 1024 );
        define( 'TGR_BREAKPOINT_TABLET', 834 );
        define( 'TGR_BREAKPOINT_MOBILE', 768 );
        define( 'TGR_XBOX_INIT_PRIORITY', 10 );
	}

	/*
	|---------------------------------------------------------------------------------------------------
	| Hooks
	|---------------------------------------------------------------------------------------------------
	*/
	private function hooks(){
		add_action( 'wp_loaded', array( $this, 'load_files_on_plugins_loaded' ) );
	}

	/*
	|---------------------------------------------------------------------------------------------------
	| General files
	|---------------------------------------------------------------------------------------------------
	*/
	public function general_files(){
        include TGR_DIR . 'includes/global-functions.php';
	}

	/*
	|---------------------------------------------------------------------------------------------------
	| Load files on "wp_loaded" hook
	|---------------------------------------------------------------------------------------------------
	*/
	public function load_files_on_plugins_loaded(){
		include TGR_DIR . 'includes/debug.php';
	}

	/*
	|---------------------------------------------------------------------------------------------------
	| Assets Loader
	|---------------------------------------------------------------------------------------------------
	*/
	public function assets_loader(){
		AssetsLoader::get_instance( $this->plugin );
	}

    /*
    |---------------------------------------------------------------------------------------------------
    | General Settings
    |---------------------------------------------------------------------------------------------------
    */
    public function general_settings(){
        $this->plugin->settings = Settings::get_instance( $this->plugin );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Prepare Editor
    |---------------------------------------------------------------------------------------------------
    */
    public function prepare_editor(){
        PrepareEditor::get_instance( $this->plugin );
        SettingsEditor::get_instance( $this->plugin );
        MemberEditor::get_instance( $this->plugin );
        TeamEditor::get_instance( $this->plugin );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Xbox Framework
    |---------------------------------------------------------------------------------------------------
    */
    public function include_xbox_framework(){
        if( ! defined( 'XBOX_HIDE_DEMO' ) ){
            define( 'XBOX_HIDE_DEMO', true );
        }
        if( file_exists( TGR_DIR . 'libs/xbox/xbox.php' ) ){
            include TGR_DIR . 'libs/xbox/xbox.php';
        }
    }

	/*
	|---------------------------------------------------------------------------------------------------
	| Ajax
	|---------------------------------------------------------------------------------------------------
	*/
	public function ajax(){
		Ajax::get_instance( $this->plugin );
	}

}