<?php namespace TeamGrid\Includes;

class PrepareEditor {
    protected static $instance = null;
    public $plugin;
    public $xbox_ids = array();

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    private function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->xbox_ids = $this->plugin->arg( 'xbox_ids' );

        $this->hooks();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Plugin hooks
    |---------------------------------------------------------------------------------------------------
    */
    private function hooks(){

        foreach( $this->plugin->post_types as $post_type ){
            add_filter( 'get_user_option_screen_layout_' . $post_type, array( $this, 'screen_layout' ) );
        }

        //add_action( 'add_meta_boxes', array( $this, 'remove_unnecessary_metaboxes' ), 99, 2 );
        add_action( 'post_edit_form_tag', array( $this, 'allow_upload_files_metaboxes' ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Screen layout to 1 column
    |---------------------------------------------------------------------------------------------------
    */
    public function screen_layout(){
        global $post;
        if( ! $post ) return 1;

        return $post->post_type == $this->plugin->arg( 'post_types', 'members' ) ? 2 : 1;
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Remove Unnecessary Metaboxes
    |---------------------------------------------------------------------------------------------------
    | http://stackoverflow.com/questions/28279831/how-do-i-remove-all-the-metaboxes-for-a-custom-post-type
    */
    public function remove_unnecessary_metaboxes( $post_type, $post ){
        global $wp_meta_boxes;

        if( ! in_array( $post_type, $this->plugin->post_types ) ){
            return false;
        }

        /** Metaboxes that will not be removed */
        $exceptions = array(
            'submitdiv',//Publish/update
            'pageparentdiv',//Post Attributes (Page Templates)
            $this->plugin->arg( 'taxonomies', 'filters' ).'div',
            $this->plugin->arg( 'taxonomies', 'fields' ).'div',
            $this->plugin->arg( 'taxonomies', 'skills' ).'div',
        );
        $exceptions = array_merge( $exceptions, array_values( $this->xbox_ids ) );

        /** Loop through each page key of the '$wp_meta_boxes' global... */
        if( ! empty( $wp_meta_boxes ) ) : foreach( $wp_meta_boxes as $page => $page_boxes ) :
            /** Loop through each contect... */
            if( ! empty( $page_boxes ) ) : foreach( $page_boxes as $context => $box_context ) :
                /** Loop through each type of meta box... */
                if( ! empty( $box_context ) ) : foreach( $box_context as $box_type ) :
                    /** Loop through each individual box... */
                    if( ! empty( $box_type ) ) : foreach( $box_type as $id => $box ) :
                        /** Check to see if the meta box should be removed... */
                        if( ! in_array( $id, $exceptions ) ) :
                            /** Remove the meta box */
                            remove_meta_box( $id, $page, $context );
                        endif;
                    endforeach;
                    endif;
                endforeach;
                endif;
            endforeach;
            endif;
        endforeach;
        endif;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Allows uploading of files from metaboxes. Required for Import function
    |---------------------------------------------------------------------------------------------------
    */
    public function allow_upload_files_metaboxes(){
        global $post;
        if( !in_array( $post->post_type, $this->plugin->post_types ) ){
            return;
        }
        echo ' enctype="multipart/form-data" encoding="multipart/form-data"';
    }

}